@@ -33,6 +33,7 @@
 #ifdef SOURCE_SADF
 #include "rndr_stats.h"
 #include "xml_stats.h"
+#include "json_stats.h"
 #endif
 
 /*
@@ -83,6 +84,7 @@ struct activity cpu_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_cpu_stats,
 	.f_xml_print	= xml_print_cpu_stats,
+	.f_json_print	= json_print_cpu_stats,
 	.hdr_line	= "CPU;%user;%nice;%system;%iowait;%steal;%idle|"
 		          "CPU;%usr;%nice;%sys;%iowait;%steal;%irq;%soft;%guest;%idle",
 	.name		= "A_CPU",
@@ -114,6 +116,7 @@ struct activity pcsw_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pcsw_stats,
 	.f_xml_print	= xml_print_pcsw_stats,
+	.f_json_print	= json_print_pcsw_stats,
 	.hdr_line	= "proc/s;cswch/s",
 	.name		= "A_PCSW",
 #endif
@@ -144,6 +147,7 @@ struct activity irq_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_irq_stats,
 	.f_xml_print	= xml_print_irq_stats,
+	.f_json_print	= json_print_irq_stats,
 	.hdr_line	= "INTR;intr/s",
 	.name		= "A_IRQ",
 #endif
@@ -174,6 +178,7 @@ struct activity swap_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_swap_stats,
 	.f_xml_print	= xml_print_swap_stats,
+	.f_json_print	= json_print_swap_stats,
 	.hdr_line	= "pswpin/s;pswpout/s",
 	.name		= "A_SWAP",
 #endif
@@ -204,6 +209,7 @@ struct activity paging_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_paging_stats,
 	.f_xml_print	= xml_print_paging_stats,
+	.f_json_print	= json_print_paging_stats,
 	.hdr_line	= "pgpgin/s;pgpgout/s;fault/s;majflt/s;"
 		          "pgfree/s;pgscank/s;pgscand/s;pgsteal/s;%vmeff",
 	.name		= "A_PAGE",
@@ -235,6 +241,7 @@ struct activity io_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_io_stats,
 	.f_xml_print	= xml_print_io_stats,
+	.f_json_print	= json_print_io_stats,
 	.hdr_line	= "tps;rtps;wtps;bread/s;bwrtn/s",
 	.name		= "A_IO",
 #endif
@@ -265,6 +272,7 @@ struct activity memory_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_memory_stats,
 	.f_xml_print	= xml_print_memory_stats,
+	.f_json_print	= json_print_memory_stats,
 	.hdr_line	= "frmpg/s;bufpg/s;campg/s|"
 		          "kbmemfree;kbmemused;%memused;kbbuffers;kbcached;kbcommit;%commit;kbactive;kbinact|"
 		          "kbswpfree;kbswpused;%swpused;kbswpcad;%swpcad",
@@ -297,6 +305,7 @@ struct activity ktables_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_ktables_stats,
 	.f_xml_print	= xml_print_ktables_stats,
+	.f_json_print	= json_print_ktables_stats,
 	.hdr_line	= "dentunusd;file-nr;inode-nr;pty-nr",
 	.name		= "A_KTABLES",
 #endif
@@ -327,6 +336,7 @@ struct activity queue_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_queue_stats,
 	.f_xml_print	= xml_print_queue_stats,
+	.f_json_print	= json_print_queue_stats,
 	.hdr_line	= "runq-sz;plist-sz;ldavg-1;ldavg-5;ldavg-15;blocked",
 	.name		= "A_QUEUE",
 #endif
@@ -357,6 +367,7 @@ struct activity serial_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_serial_stats,
 	.f_xml_print	= xml_print_serial_stats,
+	.f_json_print	= json_print_serial_stats,
 	.hdr_line	= "TTY;rcvin/s;txmtin/s;framerr/s;prtyerr/s;brk/s;ovrun/s",
 	.name		= "A_SERIAL",
 #endif
@@ -387,6 +398,7 @@ struct activity disk_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_disk_stats,
 	.f_xml_print	= xml_print_disk_stats,
+	.f_json_print	= json_print_disk_stats,
 	.hdr_line	= "DEV;tps;rd_sec/s;wr_sec/s;avgrq-sz;avgqu-sz;await;svctm;%util",
 	.name		= "A_DISK",
 #endif
@@ -417,6 +429,7 @@ struct activity net_dev_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_dev_stats,
 	.f_xml_print	= xml_print_net_dev_stats,
+	.f_json_print	= json_print_net_dev_stats,
 	.hdr_line	= "IFACE;rxpck/s;txpck/s;rxkB/s;txkB/s;rxcmp/s;txcmp/s;rxmcst/s",
 	.name		= "A_NET_DEV",
 #endif
@@ -447,6 +460,7 @@ struct activity net_edev_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_edev_stats,
 	.f_xml_print	= xml_print_net_edev_stats,
+	.f_json_print	= json_print_net_edev_stats,
 	.hdr_line	= "IFACE;rxerr/s;txerr/s;coll/s;rxdrop/s;txdrop/s;"
 		          "txcarr/s;rxfram/s;rxfifo/s;txfifo/s",
 	.name		= "A_NET_EDEV",
@@ -478,6 +492,7 @@ struct activity net_nfs_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_nfs_stats,
 	.f_xml_print	= xml_print_net_nfs_stats,
+	.f_json_print	= json_print_net_nfs_stats,
 	.hdr_line	= "call/s;retrans/s;read/s;write/s;access/s;getatt/s",
 	.name		= "A_NET_NFS",
 #endif
@@ -508,6 +523,7 @@ struct activity net_nfsd_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_nfsd_stats,
 	.f_xml_print	= xml_print_net_nfsd_stats,
+	.f_json_print	= json_print_net_nfsd_stats,
 	.hdr_line	= "scall/s;badcall/s;packet/s;udp/s;tcp/s;hit/s;miss/s;"
 		          "sread/s;swrite/s;saccess/s;sgetatt/s",
 	.name		= "A_NET_NFSD",
@@ -539,6 +555,7 @@ struct activity net_sock_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_sock_stats,
 	.f_xml_print	= xml_print_net_sock_stats,
+	.f_json_print	= json_print_net_sock_stats,
 	.hdr_line	= "totsck;tcpsck;udpsck;rawsck;ip-frag;tcp-tw",
 	.name		= "A_NET_SOCK",
 #endif
@@ -569,6 +586,7 @@ struct activity net_ip_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_ip_stats,
 	.f_xml_print	= xml_print_net_ip_stats,
+	.f_json_print	= json_print_net_ip_stats,
 	.hdr_line	= "irec/s;fwddgm/s;idel/s;orq/s;asmrq/s;asmok/s;fragok/s;fragcrt/s",
 	.name		= "A_NET_IP",
 #endif
@@ -599,6 +617,7 @@ struct activity net_eip_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eip_stats,
 	.f_xml_print	= xml_print_net_eip_stats,
+	.f_json_print	= json_print_net_eip_stats,
 	.hdr_line	= "ihdrerr/s;iadrerr/s;iukwnpr/s;idisc/s;odisc/s;onort/s;asmf/s;fragf/s",
 	.name		= "A_NET_EIP",
 #endif
@@ -629,6 +648,7 @@ struct activity net_icmp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_icmp_stats,
 	.f_xml_print	= xml_print_net_icmp_stats,
+	.f_json_print	= json_print_net_icmp_stats,
 	.hdr_line	= "imsg/s;omsg/s;iech/s;iechr/s;oech/s;oechr/s;itm/s;itmr/s;otm/s;"
 		          "otmr/s;iadrmk/s;iadrmkr/s;oadrmk/s;oadrmkr/s",
 	.name		= "A_NET_ICMP",
@@ -660,6 +680,7 @@ struct activity net_eicmp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eicmp_stats,
 	.f_xml_print	= xml_print_net_eicmp_stats,
+	.f_json_print	= json_print_net_eicmp_stats,
 	.hdr_line	= "ierr/s;oerr/s;idstunr/s;odstunr/s;itmex/s;otmex/s;"
 		          "iparmpb/s;oparmpb/s;isrcq/s;osrcq/s;iredir/s;oredir/s",
 	.name		= "A_NET_EICMP",
@@ -691,6 +712,7 @@ struct activity net_tcp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_tcp_stats,
 	.f_xml_print	= xml_print_net_tcp_stats,
+	.f_json_print	= json_print_net_tcp_stats,
 	.hdr_line	= "active/s;passive/s;iseg/s;oseg/s",
 	.name		= "A_NET_TCP",
 #endif
@@ -721,6 +743,7 @@ struct activity net_etcp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_etcp_stats,
 	.f_xml_print	= xml_print_net_etcp_stats,
+	.f_json_print	= json_print_net_etcp_stats,
 	.hdr_line	= "atmptf/s;estres/s;retrans/s;isegerr/s;orsts/s",
 	.name		= "A_NET_ETCP",
 #endif
@@ -751,6 +774,7 @@ struct activity net_udp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_udp_stats,
 	.f_xml_print	= xml_print_net_udp_stats,
+	.f_json_print	= json_print_net_udp_stats,
 	.hdr_line	= "idgm/s;odgm/s;noport/s;idgmerr/s",
 	.name		= "A_NET_UDP",
 #endif
@@ -781,6 +805,7 @@ struct activity net_sock6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_sock6_stats,
 	.f_xml_print	= xml_print_net_sock6_stats,
+	.f_json_print	= json_print_net_sock6_stats,
 	.hdr_line	= "tcp6sck;udp6sck;raw6sck;ip6-frag",
 	.name		= "A_NET_SOCK6",
 #endif
@@ -811,6 +836,7 @@ struct activity net_ip6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_ip6_stats,
 	.f_xml_print	= xml_print_net_ip6_stats,
+	.f_json_print	= json_print_net_ip6_stats,
 	.hdr_line	= "irec6/s;fwddgm6/s;idel6/s;orq6/s;asmrq6/s;asmok6/s;"
 			  "imcpck6/s;omcpck6/s;fragok6/s;fragcr6/s",
 	.name		= "A_NET_IP6",
@@ -842,6 +868,7 @@ struct activity net_eip6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eip6_stats,
 	.f_xml_print	= xml_print_net_eip6_stats,
+	.f_json_print	= json_print_net_eip6_stats,
 	.hdr_line	= "ihdrer6/s;iadrer6/s;iukwnp6/s;i2big6/s;idisc6/s;odisc6/s;"
 			  "inort6/s;onort6/s;asmf6/s;fragf6/s;itrpck6/s",
 	.name		= "A_NET_EIP6",
@@ -873,6 +900,7 @@ struct activity net_icmp6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_icmp6_stats,
 	.f_xml_print	= xml_print_net_icmp6_stats,
+	.f_json_print	= json_print_net_icmp6_stats,
 	.hdr_line	= "imsg6/s;omsg6/s;iech6/s;iechr6/s;oechr6/s;igmbq6/s;igmbr6/s;ogmbr6/s;"
 			  "igmbrd6/s;ogmbrd6/s;irtsol6/s;ortsol6/s;irtad6/s;inbsol6/s;onbsol6/s;"
 			  "inbad6/s;onbad6/s",
@@ -905,6 +933,7 @@ struct activity net_eicmp6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_eicmp6_stats,
 	.f_xml_print	= xml_print_net_eicmp6_stats,
+	.f_json_print	= json_print_net_eicmp6_stats,
 	.hdr_line	= "ierr6/s;idtunr6/s;odtunr6/s;itmex6/s;otmex6/s;"
 		          "iprmpb6/s;oprmpb6/s;iredir6/s;oredir6/s;ipck2b6/s;opck2b6/s",
 	.name		= "A_NET_EICMP6",
@@ -936,6 +965,7 @@ struct activity net_udp6_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_net_udp6_stats,
 	.f_xml_print	= xml_print_net_udp6_stats,
+	.f_json_print	= json_print_net_udp6_stats,
 	.hdr_line	= "idgm6/s;odgm6/s;noport6/s;idgmer6/s",
 	.name		= "A_NET_UDP6",
 #endif
@@ -966,6 +996,7 @@ struct activity pwr_cpufreq_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_cpufreq_stats,
 	.f_xml_print	= xml_print_pwr_cpufreq_stats,
+	.f_json_print	= json_print_pwr_cpufreq_stats,
 	.hdr_line	= "CPU;MHz",
 	.name		= "A_PWR_CPUFREQ",
 #endif
@@ -996,6 +1027,7 @@ struct activity pwr_fan_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_fan_stats,
 	.f_xml_print	= xml_print_pwr_fan_stats,
+	.f_json_print	= json_print_pwr_fan_stats,
 	.hdr_line	= "FAN;DEVICE;rpm;drpm",
 	.name		= "A_PWR_FAN",
 #endif
@@ -1026,6 +1058,7 @@ struct activity pwr_temp_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_temp_stats,
 	.f_xml_print	= xml_print_pwr_temp_stats,
+	.f_json_print	= json_print_pwr_temp_stats,
 	.hdr_line	= "TEMP;DEVICE;degC;%temp",
 	.name		= "A_PWR_TEMP",
 #endif
@@ -1056,6 +1089,7 @@ struct activity pwr_in_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_in_stats,
 	.f_xml_print	= xml_print_pwr_in_stats,
+	.f_json_print	= json_print_pwr_in_stats,
 	.hdr_line	= "IN;DEVICE;inV;%in",
 	.name		= "A_PWR_IN",
 #endif
@@ -1086,6 +1120,7 @@ struct activity huge_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_huge_stats,
 	.f_xml_print	= xml_print_huge_stats,
+	.f_json_print	= json_print_huge_stats,
 	.hdr_line	= "kbhugfree;kbhugused;%hugused",
 	.name		= "A_HUGE",
 #endif
@@ -1116,6 +1151,7 @@ struct activity pwr_wghfreq_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_wghfreq_stats,
 	.f_xml_print	= xml_print_pwr_wghfreq_stats,
+	.f_json_print	= json_print_pwr_wghfreq_stats,
 	.hdr_line	= "CPU;wghMHz",
 	.name		= "A_PWR_WGHFREQ",
 #endif
@@ -1146,6 +1182,7 @@ struct activity pwr_usb_act = {
 #ifdef SOURCE_SADF
 	.f_render	= render_pwr_usb_stats,
 	.f_xml_print	= xml_print_pwr_usb_stats,
+	.f_json_print	= json_print_pwr_usb_stats,
 	.hdr_line	= "manufact;product;BUS;idvendor;idprod;maxpower",
 	.name		= "A_PWR_USB",
 #endif
